@@ -788,6 +788,7 @@ class TestUniversalEvent:
         async def main():
             evt = UniversalEvent()
             t1 = await spawn(event_waiter, evt)
+            await sleep(0.05)
             t2 = threading.Thread(target=asyncio.run, args=[event_setter(evt, 1)])
             t2.start()
             await t1.join()
@@ -842,10 +843,113 @@ class TestUniversalEvent:
             False
         ]
 
+    def test_uevent_get_wait_timeout(self, kernel):
+        results = []
+        async def event_setter(evt, seconds):
+            results.append('sleep')
+            await sleep(seconds)
+            results.append('event_set')
+            await evt.set()
 
+        async def event_waiter(evt, seconds):
+            try:
+                results.append('wait_start')
+                await timeout_after(seconds, evt.wait())
+                results.append('wait_done')
+                results.append(evt.is_set())
+            except TaskTimeout:
+                results.append('wait_timeout')
+                results.append(evt.is_set())
+
+        async def main():
+            evt = UniversalEvent()
+            t1 = await spawn(event_waiter, evt, 1)
+            t2 = await spawn(event_setter, evt, 2)
+            await t1.join()
+            await t2.join()
+
+        kernel.run(main())
+        assert results == [
+            'wait_start',
+            'sleep',
+            'wait_timeout',
+            False,
+            'event_set',
+        ]
+        
+class TestResult:
+    def test_value(self, kernel):
+        
+        async def work(x, y, r):
+            await r.set_value(x+y)
+
+        async def main():
+            r = Result()
+            await spawn(work, 2, 3, r)
+            assert await r.unwrap() == 5
+
+        kernel.run(main)
+
+    def test_error(self, kernel):
+        async def work(x, y, r):
+            try:
+                await r.set_value(x+y)
+            except Exception as err:
+                await r.set_exception(err)
+
+        async def main():
+            r = Result()
+            await spawn(work, 2, "3", r)
+            with pytest.raises(TypeError):
+                await r.unwrap()
+
+        kernel.run(main)
+
+        
+class TestUniversalResult:
+    def test_universal_value(self, kernel):
+        
+        def work(x, y, r):
+            r.set_value(x+y)
+
+        async def main(r1, r2):
+            value = await r1.unwrap()
+            await r2.set_value(value)
+
+        r1 = UniversalResult()
+        r2 = UniversalResult()
+        r3 = UniversalResult()
+        threading.Thread(target=work, args=[2,3,r1]).start()
+        threading.Thread(target=asyncio.run, args=[main(r1, r2)]).start()
+        kernel.run(main, r2, r3)
+        assert r3.unwrap() == 5
+
+    def test_universal_error(self, kernel):
+        
+        def work(x, y, r):
+            try:
+                r.set_value(x+y)
+            except Exception as err:
+                r.set_exception(err)
+
+        async def main(r1, r2):
+            try:
+                value = await r1.unwrap()
+                await r2.set_value(value)
+            except Exception as err:
+                await r2.set_exception(err)
+
+        r1 = UniversalResult()
+        r2 = UniversalResult()
+        r3 = UniversalResult()
+        threading.Thread(target=work, args=[2,"3",r1]).start()
+        threading.Thread(target=asyncio.run, args=[main(r1, r2)]).start()
+        kernel.run(main, r2, r3)
+        with pytest.raises(TypeError):
+            val = r3.unwrap()
 
 
 def test_repr():
     # For test coverage
-    for cls in [Lock, Event, Semaphore, Condition, RLock, UniversalEvent ]:
+    for cls in [Lock, Event, Semaphore, Condition, RLock, UniversalEvent, Result, UniversalResult ]:
         repr(cls())
